// (C) Copyright 1999 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
// Created by: Jorge Lopez
//             ObjectDBX Development
//             ADF / Engineering
//             Autodesk, Inc.

//-----------------------------------------------------------------------------
#include "StdAfx.h"

#include "axdockbar.h"
#include "accont_i.c"

//-----------------------------------------------------------------------------
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//-----------------------------------------------------------------------------
IMPLEMENT_DYNAMIC(CAcUiContainerBar, CAcUiDockControlBar)

BEGIN_MESSAGE_MAP(CAcUiContainerBar, CAcUiDockControlBar)
	//{{AFX_MSG_MAP(CAcUiContainerBar)
    ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//-----------------------------------------------------------------------------
CAcUiContainerBar::CAcUiContainerBar(REFCLSID clsid) : CAcUiDockControlBar () 
{
	//{{AFX_DATA_INIT(CAcUiContainerBar)
	//}}AFX_DATA_INIT
    
    SetToolID(const_cast<CLSID*>(&clsid));
    m_EnableDockingCalled = false;
    m_PreferredDockPosition = 0L;
    m_pPreferredFloatingRect = NULL;
}

CAcUiContainerBar::~CAcUiContainerBar()
{
    delete m_pPreferredFloatingRect;
}

BOOL CAcUiContainerBar::Create(CWnd*pParent, LPCSTR lpszTitle) 
{
    CString strWndClass ;
	strWndClass =AfxRegisterWndClass (CS_DBLCLKS, LoadCursor (NULL, IDC_ARROW)) ;
    CRect rect (0, 0, 250, 200) ;
    if ( !CAcUiDockControlBar::Create (
				strWndClass,
				lpszTitle,
				WS_VISIBLE | WS_CHILD | WS_CLIPCHILDREN,
				rect,
				pParent, 0)
		)
        return (FALSE) ;

	//----- TODO: Add your code here
	
	return (TRUE) ;
}

int CAcUiContainerBar::OnCreate (LPCREATESTRUCT lpCreateStruct) 
{
    if ( CAcUiDockControlBar::OnCreate (lpCreateStruct) == -1 )
        return (-1) ;

    CRect rect;
    GetClientRect(&rect);

    //----- TODO: Add your code here
    if (!m_OCX.CreateControl(*GetToolID(), NULL, WS_VISIBLE,
                             rect, this, 10))
    {
        return (-1);
    }

    // if the control supports IRetrieveDockableContainer then use
    // it to pass in the IAcadDockableContainer object
    LPUNKNOWN pUnknown = m_OCX.GetControlUnknown();
    ASSERT(pUnknown != NULL);

    IRetrieveDockableContainer* pRetrieveDockContainer = NULL;
    HRESULT hr = pUnknown->QueryInterface(IID_IRetrieveDockableContainer,
                 reinterpret_cast<LPVOID*>(&pRetrieveDockContainer));
    if (SUCCEEDED(hr))
    {
        CAxContainerBar* pAxContainerBar = new CAxContainerBar(this);
        if (pAxContainerBar != NULL)
        {
            IAcadDockableContainer* pDockContainer = NULL;
            pDockContainer = static_cast<IAcadDockableContainer*>(pAxContainerBar);
            pDockContainer->AddRef();

            pRetrieveDockContainer->SetDockContainer(pDockContainer);

            pRetrieveDockContainer->Release();
            pDockContainer->Release();
        }
        else
        {
            pRetrieveDockContainer->Release();
            return (-1);
        }
    }

    return (0) ;
}

void CAcUiContainerBar::SizeChanged (CRect *lpRect, BOOL bFloating, int flags) 
{
	//----- TODO: Add your code here
    lpRect->DeflateRect(6, 6);      // should this be user settable?
    m_OCX.MoveWindow(lpRect);
}

void CAcUiContainerBar::ToggleDocking()
{
    ASSERT(m_pDockContext != NULL);
    m_pDockContext->ToggleDocking();
}

void CAcUiContainerBar::ToggleAllowDocking()
{
    OnAllowDock();
}
