// (C) Copyright 1999 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
// Created by: Jorge Lopez
//             ObjectDBX Development
//             ADF / Engineering
//             Autodesk, Inc.

// accont.cpp : Initialization functions
#include "StdAfx.h"
#include "StdArx.h"
#include "resource.h"
#include <afxdllx.h>

// Global variables
HINSTANCE _hdllInstance = NULL ;
DockBarMap gDockBars;

// This command registers an ARX command.
void AddCommand(const char* cmdGroup, const char* cmdInt, const char* cmdLoc,
				const int cmdFlags, const AcRxFunctionPtr cmdProc, const int idLocal = -1);

// NOTE: DO NOT edit the following lines.
//{{AFX_ARX_MSG
void InitApplication();
void UnloadApplication();
//}}AFX_ARX_MSG

// NOTE: DO NOT edit the following lines.
//{{AFX_ARX_ADDIN_FUNCS
void RegisterArxApp();
char ThisAppRegistryPath[] = { "Software\\" };
//}}AFX_ARX_ADDIN_FUNCS

////////////////////////////////////////////////////////////////////////////
//
// Define the sole extension module object.
AC_IMPLEMENT_EXTENSION_MODULE(AccontDLL);

// Now you can use the CAcModuleRecourceOverride class in
// your application to switch to the correct resource instance.
// Please see the ObjectARX Documentation for more details

/////////////////////////////////////////////////////////////////////////////
// DLL Entry Point
extern "C"
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
	if (dwReason == DLL_PROCESS_ATTACH)
	{
        _hdllInstance = hInstance;
		// Extension DLL one time initialization
		AccontDLL.AttachInstance(hInstance);
		InitAcUiDLL();
	} else if (dwReason == DLL_PROCESS_DETACH) 
    {
		// Terminate the library before destructors are called
		AccontDLL.DetachInstance();
	}
	return TRUE;    // ok
}


/////////////////////////////////////////////////////////////////////////////
// ObjectARX EntryPoint
extern "C" AcRx::AppRetCode 
acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt)
{
	switch (msg) 
    {
	case AcRx::kInitAppMsg:
		// Comment out the following line if your
		// application should be locked into memory
		acrxDynamicLinker->unlockApplication(pkt);
		acrxDynamicLinker->registerAppMDIAware(pkt);
		InitApplication();
		break;
	case AcRx::kUnloadAppMsg:
		UnloadApplication();
		break;
	}
	return AcRx::kRetOK;
}


// Init this application. Register your
// commands, reactors...
void InitApplication()
{
	// NOTE: DO NOT edit the following lines.
	//{{AFX_ARX_INIT
	AddCommand("AC_DOCK_CONT", "DOCKABLE_CONTAINER", "DC", ACRX_CMD_SESSION, ac_cont_dockable_container);
	RegisterArxApp();
	//}}AFX_ARX_INIT

	// TODO: add your initialization functions

}

// Unload this application. Unregister all objects
// registered in InitApplication.
void UnloadApplication()
{
	// NOTE: DO NOT edit the following lines.
	//{{AFX_ARX_EXIT
	acedRegCmds->removeGroup("AC_DOCK_CONT");
	//}}AFX_ARX_EXIT

	// TODO: clean up your application

    // Walk the list of running dock containers and delete each one
    DockBarMap::iterator iter;
    for(iter = gDockBars.begin(); iter != gDockBars.end(); ++iter)
    {
        delete (*iter).second;
    }
}

// This functions registers an ARX command.
// It can be used to read the localized command name
// from a string table stored in the resources.
void AddCommand(const char* cmdGroup, const char* cmdInt, const char* cmdLoc,
				const int cmdFlags, const AcRxFunctionPtr cmdProc, const int idLocal)
{
	char cmdLocRes[65];

	// If idLocal is not -1, it's treated as an ID for
	// a string stored in the resources.
	if (idLocal != -1) 
    {
		// Load strings from the string table and register the command.
		::LoadString(_hdllInstance, idLocal, cmdLocRes, 64);
		acedRegCmds->addCommand(cmdGroup, cmdInt, cmdLocRes, cmdFlags, cmdProc);
	} 
    else
		// idLocal is -1, so the 'hard coded'
		// localized function name is used.
		acedRegCmds->addCommand(cmdGroup, cmdInt, cmdLoc, cmdFlags, cmdProc);
}

// This function registers this ObjectARX Application
// on AutoCAD for demandload in the Windows Registry.
// Also, it registers the accont.tlb and all of the interfaces
// contained within.
void RegisterArxApp()
{
	ASSERT(_hdllInstance != NULL);

	char modulePath[512];
	int modulePathLength = GetModuleFileName(_hdllInstance, modulePath, sizeof(modulePath));

	//
	// Register the application.
	//
	acrxRegisterApp((AcadApp::LoadReasons) (AcadApp::kOnCommandInvocation),
	                "accont",
	                ThisAppRegistryPath, 2);

	//
	// Create the registry key for this application
	// (use the variable ThisAppRegistryPath).
	//
	HKEY rkey;
	DWORD result;
	LONG status = RegCreateKeyEx(HKEY_LOCAL_MACHINE,
	                             ThisAppRegistryPath,
	                             0, "",
	                             REG_OPTION_NON_VOLATILE,
	                             KEY_WRITE, NULL, &rkey, &result);
	if (ERROR_SUCCESS != status)
		return;

	HKEY prodKey;
	status = RegCreateKeyEx(rkey, "accont", 0, "",
	                        REG_OPTION_NON_VOLATILE,
	                        KEY_WRITE, NULL, &prodKey, &result);
	RegCloseKey(rkey);
	if (ERROR_SUCCESS != status)
		return;

	// Create the 'Loader' key.
	HKEY loaderKey;
	status = RegCreateKeyEx(prodKey, "Loader", 0, "",
	                        REG_OPTION_NON_VOLATILE,
	                        KEY_WRITE, NULL, &loaderKey, &result);
	if (ERROR_SUCCESS == status) 
    {
		if (modulePathLength > 0)
			RegSetValueEx(loaderKey, "MODULE", 0, REG_SZ,
			              (const unsigned char*)modulePath, modulePathLength + 1);
		RegCloseKey(loaderKey);
	}

	// Create the 'Name' key.
	HKEY nameKey;
	status = RegCreateKeyEx(prodKey, "Name", 0, "",
	                        REG_OPTION_NON_VOLATILE,
	                        KEY_WRITE, NULL, &nameKey, &result);
	if (ERROR_SUCCESS == status) 
    {
		RegSetValueEx(nameKey, "accont", 0, REG_SZ,
		              (const unsigned char*) "accont", 7);
		RegCloseKey(nameKey);
	}

	//
	// Register the command groups.
	//
	HKEY groupKey;
	status = RegCreateKeyEx(prodKey, "Groups", 0, "",
	                        REG_OPTION_NON_VOLATILE,
	                        KEY_WRITE, NULL, &groupKey, &result);
	if (ERROR_SUCCESS == status) 
    {
		RegSetValueEx(groupKey, "AC_DOCK_CONT", 0, REG_SZ,
		              (const unsigned char*) "AC_DOCK_CONT", 13);
		RegCloseKey(groupKey);
	}

	//
	// Register the commands.
	//
	HKEY cmdKey;
	status = RegCreateKeyEx(prodKey, "Commands", 0, "",
	                        REG_OPTION_NON_VOLATILE,
	                        KEY_WRITE, NULL, &cmdKey, &result);
	if (ERROR_SUCCESS == status) {
		// If you like you can store the localized command name in
		// the Windows Registry and load it from there.
		RegSetValueEx(cmdKey, "DC", 0, REG_SZ,
		              (const unsigned char*) "DC", 3);
		RegCloseKey(cmdKey);
	}

	//
	// Finished with demand loading.
	//
	RegCloseKey(prodKey);

	//
	// Register the type library.
	//
	if (modulePathLength > 0)
	{
		// Change the file extension from .ARX to .TLB
		CString tlbName(modulePath);
		ASSERT(tlbName.Right(4).CompareNoCase(".ARX") == 0);

		tlbName = tlbName.Left(tlbName.GetLength() - 3);
		tlbName += "tlb";

		USES_CONVERSION;
		ITypeLib* pTypeLib = NULL;

		HRESULT hr = LoadTypeLib(A2W(tlbName), &pTypeLib);
		if (SUCCEEDED(hr))
		{
			hr = RegisterTypeLib(pTypeLib, A2W(tlbName), NULL);
			ASSERT(SUCCEEDED(hr));

			pTypeLib->Release();
		}
	}
}

